#!/bin/bash

export LC_ALL=C

WORKDIR=./kernel_work
# Input file
KERNEL=$WORKDIR/kernel
HEADER=$WORKDIR/header
FOOTER=$WORKDIR/footer
# Output file
OUTPUT_FILE=KERNEL.tar

# [ERROR] Required dir/file not found
if [ ! -d $WORKDIR ]; then
  printf "[ERROR] \"$WORKDIR\" Directory not found\n"
  exit 1
fi
[ -z $KERNEL ] && printf "[ERROR] \"$KERNEL\" File not found\n" && exit 1
[ -z $HEADER ] && printf "[ERROR] \"$HEADER\" File not found\n" && exit 1
[ -z $FOOTER ] && printf "[ERROR] \"$FOOTER\" File not found\n" && exit 1

### Main Section

# Required LZMA size
size_header=$(stat -c %s $HEADER)
pos_footer=`od -j416 -A n -t u4 -N 4 $HEADER | sed 's/ //g'`
size_lzma_req=$((pos_footer-size_header))
printf "Required LZMA size=$size_lzma_req\n"

# Compress(lzma) kernel to newkernel.lzma
printf "LZMA compress..."
cat $KERNEL | lzma -9 > $WORKDIR/newkernel.lzma

# Check the size
newsize=$(stat -c %s $WORKDIR/newkernel.lzma)
printf "newkernel.lzma was created with $newsize bytes\n"
[ $newsize -gt $size_lzma_req ] && printf "[ERROR] newkernel.lzma too big, OVER $size_lzma_req bytes\n" && exit 1

# Resize newkernel.lzma
if [ $newsize -eq $size_lzma_req ]; then
  cp $WORKDIR/newkernel.lzma $WORKDIR/newkernel_resize
else
  dd if=$WORKDIR/newkernel.lzma of=$WORKDIR/newkernel_resize bs=$size_lzma_req conv=sync 2>/dev/null >/dev/null
  printf "Resized $newsize to $(stat -c %s $WORKDIR/newkernel_resize) bytes\n"
fi

# Combine header, resized-kernel, footer to zImage
cat $HEADER $WORKDIR/newkernel_resize $FOOTER > $WORKDIR/zImage

# Compress(tar) zImage to KERNEL.tar
cd $WORKDIR
tar -cf $OUTPUT_FILE zImage
cd ..

printf "\"$WORKDIR/$OUTPUT_FILE\" has been created\n"

# Remove work files
rm -f $WORKDIR/newkernel.lzma
rm -f $WORKDIR/newkernel_resize
rm -f $WORKDIR/zImage

exit 0