#!/bin/bash

export LC_ALL=C

# [ERROR] No argument
[ -z $1 ] && printf "[ERROR] No argument\n" && exit 1

WORKDIR=./kernel_work
# Input file
ZIMAGE=$1
# Output file
KERNEL=$WORKDIR/kernel
HEADER=$WORKDIR/header
FOOTER=$WORKDIR/footer

# [ERROR] zImage not found
[ ! -e $ZIMAGE ] && printf "[ERROR] $ZIMAGE not found\n" && exit 1

# Make work-dir
#[ -d $WORKDIR ] || `mkdir $WORKDIR`
rm -dfr $WORKDIR
mkdir $WORKDIR

### Main Section

# zImage size
zImage_size=$(stat -c %s $ZIMAGE)
printf "zImage size=$zImage_size\n"

# Footer position
pos_footer=`od -j416 -A n -t u4 -N 4 $ZIMAGE | sed 's/ //g'`
printf "Footer position=$pos_footer\n"
[ $pos_footer -ge $zImage_size ] && printf "[ERROR] Footer position is abnormal\n" && exit 1

# Footer size
size_footer=$((zImage_size-pos_footer))
printf "Footer size=$size_footer\n"

# LZMA position step-1
pos_lzma=`grep -P -a -b -m 1 --only-matching 'LZMA data is corrupt' $ZIMAGE | cut -f 1 -d : 2>/dev/null | awk '(int($0)<10000){print $0;exit}'`
[ -z $pos_lzma ] && printf "[ERROR] lzma-image not found in $ZIMAGE\n (step-1)" && exit 1
dd if=$ZIMAGE of=$WORKDIR/work.1 bs=$pos_lzma skip=1 2>/dev/null >/dev/null

# LZMA position step-2
pos2=`grep -P -a -b -m 1 --only-matching '\x5D' $WORKDIR/work.1 | cut -f 1 -d : 2>/dev/null | awk '(int($0)<50){print $0;exit}'`
[ -z $pos2 ] && printf "[ERROR] lzma-image not found in $ZIMAGE\n (step-2)" && exit 1
pos_lzma=$((pos_lzma+pos2))
printf "LZMA position (Header size)=$pos_lzma\n"

# LZMA size
size_lzma=$((pos_footer-pos_lzma))
printf "LZMA size=$size_lzma (Footer position - LZMA position)\n"

# Cut each part from zImager
dd if=$ZIMAGE of=$HEADER bs=$pos_lzma count=1 2>/dev/null >/dev/null
dd if=$ZIMAGE of=$WORKDIR/work.2 bs=$pos_lzma skip=1 2>/dev/null >/dev/null
dd if=$WORKDIR/work.2 of=$WORKDIR/kernel.lzma bs=$size_lzma count=1 2>/dev/null >/dev/null
dd if=$ZIMAGE of=$FOOTER bs=$pos_footer skip=1 2>/dev/null >/dev/null

# Extract LZMA
cat $WORKDIR/kernel.lzma | unlzma -dqc > $KERNEL 2>/dev/null
printf "Extract to \"$KERNEL\"\n"

# Remove work files
rm -f $WORKDIR/work.1
rm -f $WORKDIR/work.2
rm -f $WORKDIR/kernel.lzma

exit 0